// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using System.Numerics;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // このサンプルは、既存のPDFの全ページにシンプルなテキストの透かしのような
    // オーバーレイを追加する方法を示しています。
    public class AddWatermark
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            using var fs = File.OpenRead(Path.Combine("Resources", "PDFs", "SlidePages.pdf"));
                doc.Load(fs);
                foreach (var page in doc.Pages)
                {
                    var g = page.Graphics;

                    // 透かしの描画に使用されるテキストレイアウト。
                    var tl = g.CreateTextLayout();
                    tl.Append("DioDocs for PDF Demo");
                    tl.DefaultFormat.Font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "calibrib.ttf"));
                    tl.DefaultFormat.FontSize = g.Resolution;
                    // 半透明の色。
                    tl.DefaultFormat.ForeColor = Color.FromArgb(128, Color.Yellow);
                    tl.DefaultFormat.GlyphAdvanceFactor = 1.5f;
                    tl.PerformLayout();

                    // 回転角度（ラジアン） - ページの左下から右上の角まで。
                    var angle = -Math.Asin(g.CanvasSize.Width / g.CanvasSize.Height);
                    // ページの中央。
                    var center = new PointF(g.CanvasSize.Width / 2, g.CanvasSize.Height / 2);
                    // テキストサイズを基にオフセットを追加。
                    var delta = new PointF(
                        (float)((tl.ContentWidth * Math.Cos(angle) - tl.ContentHeight * Math.Sin(angle)) / 2),
                        (float)((tl.ContentWidth * Math.Sin(angle) + tl.ContentHeight * Math.Cos(angle)) / 2));

                    // 透かしテキストをページの中央に斜めに描画します。
                    // （Matrixの変換は最後から最初へ適用されます）
                    g.Transform =
                        Matrix3x2.CreateRotation((float)angle) *
                        Matrix3x2.CreateTranslation(center.X - delta.X, center.Y - delta.Y);

                    g.DrawTextLayout(tl, PointF.Empty);
                    g.Transform = Matrix3x2.Identity;
                }
                doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
